@@ -65,15 +65,15 @@ nb.loglikelyhood <- function(hist.table, zero.items, size, mu)
 ### EM algorithm to fit the histogram with a negative binomial distribution
 ### hist only includes information for observation
 ### the number of unobserved items is missing data
-preseqR.ztnb.em <- function(hist, size=SIZE.INIT, mu=MU.INIT, header=FALSE)
+preseqR.ztnb.em <- function(hist, size=SIZE.INIT, mu=MU.INIT)
 {
-  hist.count <- read.hist(hist, header)
+  checking.hist(hist)
 
   ## setting the number of unobserved items as 0
   zero.prob <- exp(dnbinom(0, size = size, mu = mu, log = TRUE))
 
   ## estimate the total number of distinct items
-  observed.items <- sum(hist.count)
+  observed.items <- sum(hist[, 2])
   L <- observed.items/( 1 - zero.prob )
 
   ## expected the number of unobservations
@@ -82,27 +82,20 @@ preseqR.ztnb.em <- function(hist, size=SIZE.INIT, mu=MU.INIT, header=FALSE)
   ## convert zero.items into an integer
   zero.items <- floor(zero.items)
 
-  ## convert the count vector of a histogram into a histogram table
-  freq <- which(hist.count != 0)
-  number.items <- hist.count[freq]
-
   ## estimated mean and variance
-  m <- (freq %*% number.items) / L
-  v <- ( (freq - m)^2 %*% number.items + m^2 * zero.items )/(L - 1)
-
-  ## build a histogram table
-  hist.table <- matrix(c(freq, number.items), ncol = 2, byrow = FALSE)
+  m <- (hist[, 1] %*% hist[, 2]) / L
+  v <- ( (hist[, 1] - m)^2 %*% hist[, 2] + m^2 * zero.items )/(L - 1)
 
   ## target function f
   f <- function(x) {
-        return( -nb.loglikelyhood(hist.table, zero.items, size = x, mu = m)/L )
+        return( -nb.loglikelyhood(hist, zero.items, size = x, mu = m)/L )
   }
 
   ## derivative of f
   gr <- function(x) 
   {
     first.term <- ( digamma(x) * zero.items + 
-                    digamma(hist.table[, 1] + size) %*% hist.table[, 2] )/L
+                    digamma(hist[, 1] + size) %*% hist[, 2] )/L
     second.term <- digamma(x)
     third.term <- log(x) - log(x + m)
     result <- first.term - second.term + third.term
@@ -126,7 +119,7 @@ preseqR.ztnb.em <- function(hist, size=SIZE.INIT, mu=MU.INIT, header=FALSE)
   loglikelyhood.pre <- Inf
 
   ## zerotruncated loglikelyhood 
-  loglikelyhood <- zerotruncated.minus.log.likelyhood(hist.table, res$par, m)
+  loglikelyhood <- zerotruncated.minus.log.likelyhood(hist, res$par, m)
 
   ## EM algorithm
   while (( loglikelyhood.pre - loglikelyhood )/observed.items > TOLERANCE &&
@@ -154,8 +147,8 @@ preseqR.ztnb.em <- function(hist, size=SIZE.INIT, mu=MU.INIT, header=FALSE)
     zero.items <- floor(zero.items)
 
     ## estimated mean and variance
-    m <- (freq %*% number.items)/L
-    v <- ( (freq - m)^2 %*% number.items + m^2 * zero.items )/(L - 1)
+    m <- (hist[, 1] %*% hist[, 2])/L
+    v <- ( (hist[, 1] - m)^2 %*% hist[, 2] + m^2 * zero.items )/(L - 1)
 
 ### M step: estimate the parameters size and mu
     if (v > m) {
@@ -167,29 +160,22 @@ preseqR.ztnb.em <- function(hist, size=SIZE.INIT, mu=MU.INIT, header=FALSE)
     }
     iter <- iter + 1
     ## zerotruncated loglikelyhood
-    loglikelyhood <- zerotruncated.minus.log.likelyhood(hist.table, res$par, m)
+    loglikelyhood <- zerotruncated.minus.log.likelyhood(hist, res$par, m)
   }
   return(list(size = size, mu = mu, loglik = -loglikelyhood.pre))
 }
 
 
 ### predict the number of distinct items using EM algorithm
-### if the histogram file has a header, set header = TRUE
 ### t is the relative size to inital sample
-preseqR.ztnb.estimate <- function(hist, t, header = FALSE)
+preseqR.ztnb.estimate <- function(hist, t)
 {
-  hist.count <- read.hist(hist, header)
+  checking.hist(hist)
 
-  total.sample <- (1:length(hist.count) %*% hist.count)
-  distinct.sample <- sum(hist.count)
-
-  ## convert the count vector of a histogram into a histogram table
-  freq = which(hist.count != 0)
-  number.items = hist.count[freq]
-  hist.table = matrix(c(freq, number.items), ncol = 2, byrow = FALSE)
+  distinct <- sum(hist[, 2])
 
   ## estimate the parameters
-  opt <- preseqR.ztnb.em(hist.table)
+  opt <- preseqR.ztnb.em(hist)
   size <- opt$size
   mu <- opt$mu
 
@@ -197,33 +183,36 @@ preseqR.ztnb.estimate <- function(hist, t, header = FALSE)
   p <- 1 - dnbinom(0, size = size, mu = mu)
 
   ## L is the estimated total number of distinct items
-  L <- distinct.sample/p
+  L <- distinct/p
 
   ## update parameters of negative binomial in the experiment with size n
-  mu <- mu*(t + 1)
+  mu <- mu*t
 
   ## the probability of being sampled under the new experiment
   P <- 1 - dnbinom(0, size = size, mu = mu)
 
   ## return the expected number of new distinct species under the new experiment
-  return(L * P - distinct.sample)
+  return(L * P - distinct)
 }
 
 ## predict a complexity curve using EM algorithm
 ## ss is the step.size
 ## max.extrapoltion is the maximum value for extrapolation
 preseqR.ztnb.species.accum.curve <- function(hist, ss = NULL,
-                                             max.extrapolation = NULL,
-                                             header = FALSE)
+                                             max.extrapolation = NULL)
 {
-  hist.count <- read.hist(hist, header)
+  checking.hist(hist)
 
-  total.sample <- (1:length(hist.count) %*% hist.count)
-  distinct.sample <- sum(hist.count)
+  total.sample <- floor(hist[, 1] %*% hist[, 2])
+  distinct <- sum(hist[, 2])
 
   ## set step.size = total.sample if it is undefined
-  if (is.null(ss))
+  if (is.null(ss)) {
     ss <- total.sample
+  } else if (ss < 1) {
+    write("the step size should be at least one", stderr())
+    return(NULL)
+  }
 
   ## set max.extrapolation = 100 * ss if it is undefined
   if (is.null(max.extrapolation)) {
@@ -232,23 +221,19 @@ preseqR.ztnb.species.accum.curve <- function(hist, ss = NULL,
     max.extrapolation <- 100*total.sample
     n <- as.integer( max.extrapolation/ss )
 
+  } else if (max.extrapolation < ss) {
+    write("max.extrapolation should be no less then ss", stderr())
+    return(NULL)
   } else {
-
     # n is the number of experiments
     n <- as.integer( max.extrapolation/ss )
 
   }
 
   sample.size <- as.double(ss) * (1: n)
-  dim(sample.size) <- n
-
-  ## convert the count vector of a histogram into a histogram table
-  freq <- which(hist.count != 0)
-  number.items <- hist.count[freq]
-  hist.table <- matrix(c(freq, number.items), ncol = 2, byrow = FALSE)
 
   ## estimate parameters
-  opt <- preseqR.ztnb.em(hist.table)
+  opt <- preseqR.ztnb.em(hist)
   size <- opt$size
   mu <- opt$mu
 
@@ -256,12 +241,12 @@ preseqR.ztnb.species.accum.curve <- function(hist, ss = NULL,
   p <- 1 - dnbinom(0, size = size, mu = mu)
 
   ## L is the estimated total number of distinct items
-  L <- distinct.sample/p
+  L <- distinct/p
 
   ## estimate the item being sampled under new experiments with different size
-  t = sample.size/as.double(total.sample)
+  t = sample.size/total.sample
   dim(t) = length(t)
-  P = apply(t, 1, function(x) 1 - dnbinom(0, size, mu = x * mu))
+  P = apply(t, 1, function(x) {1 - dnbinom(0, size, mu = x * mu)})
   yield.estimates = L*P
 
   ## combine sample.size and yield.estimates into matrix
